<?php
/*
 * @copyright Copyright (c) 2024 AltumCode (https://altumcode.com/)
 *
 * This software is exclusively sold through https://altumcode.com/ by the AltumCode author.
 * Downloading this product from any other sources and running it without a proper license is illegal,
 *  except the official ones linked from https://altumcode.com/.
 */

namespace Altum\controllers;

use Altum\Alerts;
use Altum\Meta;
use Altum\Router;
use Altum\Title;

class ToolsAreaConverter extends Controller {

    public function index() {
        $url = $this->params[0];
        $tool_key = str_replace('-', '_', $url);

        /* Process the URL */
        $exploded = explode('-to-', $url);
        $from = str_replace('-', '_', $exploded[0]);
        $to = str_replace('-', '_', $exploded[1]);

        /* Process everything else */
        if(!settings()->tools->available_tools->{$tool_key}) {
            redirect();
        }

        if(!$this->user->plan_settings->enabled_tools->{$tool_key}) {
            Alerts::add_info(l('global.info_message.plan_feature_no_access'));
            redirect();
        }

        if(string_starts_with('tools/', \Altum\Router::$original_request)) {
            redirect(str_replace('_', '-', $tool_key), true, 301);
        }

        /* Add a new view to the page */
        $cookie_name = 't_statistics_' . $tool_key;
        if(!isset($_COOKIE[$cookie_name])) {
            setcookie($cookie_name, (int) true, time()+60*60*24*1);
            db()->onDuplicate(['total_views'], 'id');
            db()->insert('tools_usage', [
                'tool_id' => $tool_key,
                'total_views' => db()->inc(),
            ]);
        }

        $tools_usage = (new \Altum\Models\Tools())->get_tools_usage();

        /* Popular tools View */
        $view = new \Altum\View('tools/popular_tools', (array) $this);
        $this->add_view_content('popular_tools', $view->run([
            'tools_usage' => $tools_usage,
            'tools' => Router::$data['tools']
        ]));

        /* Similar tools View */
        $view = new \Altum\View('tools/similar_tools', (array) $this);
        $this->add_view_content('similar_tools', $view->run([
            'tools_usage' => $tools_usage,
            'tool' => $tool_key,
            'tools' => Router::$data['tools'],
        ]));

        /* Extra content View */
        $view = new \Altum\View('tools/extra_content', (array) $this);
        $this->add_view_content('extra_content', $view->run([
            'from' => $from,
            'to' => $to,
        ]));

        /* Meta & title */
        Title::set(sprintf(l('tools.tool_title'), sprintf(l('tools.area_converter_tools.name'), l('tools.' . $from), l('tools.' . $to))), true);
        Meta::set_description(sprintf(l('tools.area_converter_tools.description'), l('tools.' . $from), l('tools.' . $to)));
        Meta::set_keywords(sprintf(l('tools.area_converter_tools.meta_keywords'), l('tools.' . $from), l('tools.' . $to)));
        Meta::set_canonical_url();

        $data = [
            'tool' => $tool_key,
        ];

        if(!empty($_POST)) {
            $_POST['input'] = (float) $_POST['input'];
            $_POST['precision'] = (int) $_POST['precision'];

            /* Check for any errors */
            $required_fields = ['input'];
            foreach($required_fields as $field) {
                if(!isset($_POST[$field]) || (isset($_POST[$field]) && empty($_POST[$field]) && $_POST[$field] != '0')) {
                    Alerts::add_field_error($field, l('global.error_message.empty_field'));
                }
            }

            if(!\Altum\Csrf::check()) {
                Alerts::add_error(l('global.error_message.invalid_csrf_token'));
            }

            if(!Alerts::has_field_errors() && !Alerts::has_errors()) {
                $data['result'] = nr($this->convert_data_units($from, $to, $_POST['input']), $_POST['precision']);
            }
        }

        $values = [
            'input' => $_POST['input'] ?? 1,
            'precision' => $_POST['precision'] ?? 4,
        ];

        /* Conversion table */
        $data['table'] = [];
        foreach(['0.001', '0.01', '0.1', '1', '2', '3', '5', '10', '20', '30', '50', '100', '1000'] as $value) {
            $data['table'][$value] = nr($this->convert_data_units($from, $to, $value), 8, false);
        }

        /* Prepare the view */
        $data['values'] = $values;
        $data['from'] = $from;
        $data['to'] = $to;

        $view = new \Altum\View(\Altum\Plugin::get('pro-tools')->path . 'views/tools/area_converter', (array) $this, true);

        $this->add_view_content('content', $view->run($data));
    }

    private function convert_data_units($from, $to, $value) {
        $units_to_square_meters = require \Altum\Plugin::get('pro-tools')->path . 'includes/tools/area_converter_units.php';

        // Convert from the original unit
        $value_in_square_meters = $value * $units_to_square_meters[$from];

        // Convert from bits to the target unit
        return $value_in_square_meters / $units_to_square_meters[$to];
    }
}
